@@ -18,11 +18,11 @@ Text::Diff::FormattedHTML - Generate a colorful HTML diff of strings/files.
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 
 =head1 SYNOPSIS
@@ -33,7 +33,7 @@ our $VERSION = '0.01';
 
     # for strings
 
-    my $output = diff_strings($file1, $file2);
+    my $output = diff_strings( { vertical => 1 }, $file1, $file2);
 
 
     # as you might want some CSS:
@@ -52,9 +52,19 @@ Inspired on GitHub diff view.
 
 =head2 diff_files
 
+   my $html = diff_files("filename1", "filename2");
+
+C<diff_files> and C<diff_strings> support a first optional argument
+(an hash reference) where options can be set. At the moment the only
+valid option is C<vertical> that can be set to a true value, for a
+more compact table.
+
 =cut
 
 sub diff_files {
+    my $settings = {};
+    $settings = shift if ref($_[0]) eq "HASH";
+
     my ($f1, $f2) = @_;
 
     die "$f1 not available" unless -f $f1;
@@ -63,22 +73,34 @@ sub diff_files {
     my @f1 = read_file $f1;
     my @f2 = read_file $f2;
 
-    _internal_diff(\@f1, \@f2);
+    _internal_diff($settings, \@f1, \@f2);
 }
 
 =head2 diff_strings
 
+   my $html = diff_strings("string1", "string2");
+
+Compare strings. First split by newline, and then treat them as file
+content (see function above).
+
 =cut
 
 sub diff_strings {
+    my $settings = {};
+    $settings = shift if ref($_[0]) eq "HASH";
+
     my ($s1, $s2) = @_;
     my @f1 = split /\n/, $s1;
     my @f2 = split /\n/, $s2;
-    _internal_diff(\@f1, \@f2);
+    _internal_diff($settings, \@f1, \@f2);
 }
 
 =head2 diff_css
 
+   my $css = diff_css;
+
+Return the default css. You are invited to override it.
+
 =cut
 
 sub diff_css {
@@ -102,9 +124,15 @@ table.diff td:nth-child(2) {
    background-color: #deedff;
 }
 
+
+table.diff tr.change,
+table.diff tr.disc_a,
+table.diff tr.disc_b {
+   background-color: #ffffdd;
+}
+
 table.diff td:nth-child(3),
 table.diff td:nth-child(4) {
-   background-color: #fdfdfd;
    font-family: monospace;
    white-space: pre;
 }
@@ -112,7 +140,7 @@ table.diff td:nth-child(4) {
 table.diff td ins {
    padding: 2px;
    color: #009900;
-   background-color: #eeffee;
+   background-color: #ccffcc;
    text-decoration: none;
    font-weight: bold;
 }
@@ -120,7 +148,7 @@ table.diff td ins {
 table.diff td del {
    padding: 2px;
    color: #990000;
-   background-color: #ffeeee;
+   background-color: #ffcccc;
    text-decoration: none;
    font-weight: bold;
 }
@@ -139,7 +167,7 @@ sub _protect {
 }
 
 sub _internal_diff {
-    my ($sq1, $sq2) = @_;
+    my ($settings, $sq1, $sq2) = @_;
 
     my $get = sub {
         my ($l, $r) = @_;
@@ -152,7 +180,27 @@ sub _internal_diff {
 
     my ($ll, $rl);
 
-    my $line = "<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>\n";
+    my $line = sub {
+        sprintf("<tr class='%s'><td>%s</td><td>%s</td><td>%s</td><td>%s</td></tr>\n", @_);
+    };
+
+    if ($settings->{vertical}) {
+        $line = sub {
+            my $out = "";
+            my ($class, $ln, $rn, $l, $r) = @_;
+            if ($l eq $r) {
+                $out .= sprintf("<tr class='%s'><td>%s</td><td>%s</td><td>%s</td></tr>\n",
+                                $class, $ln, $rn, $l);
+            } else {
+                $l and $out .= sprintf("<tr class='%s'><td>%s</td><td></td><td>%s</td></tr>\n",
+                                       $class, $ln, $l);
+                $r and $out .= sprintf("<tr class='%s'><td></td><td>%s</td><td>%s</td></tr>\n",
+                                       $class, $rn, $r);
+            }
+            $out
+        }
+    }
+
     my $out = "<table class='diff'>\n";
 
     traverse_balanced $sq1, $sq2,
@@ -160,17 +208,17 @@ sub _internal_diff {
        MATCH     => sub {
            my ($l, $r) = $get->(@_);
            ++$ll; ++$rl;
-           $out .= sprintf $line => $ll, $rl, _protect($l), _protect($r);
+           $out .= $line->('match', $ll, $rl, _protect($l), _protect($r));
        },
        DISCARD_A => sub {
            my ($l, $r) = $get->(@_);
            ++$ll;
-           $out .= sprintf $line => $ll, '', _protect($l), '';
+           $out .= $line->('disc_a', $ll, '', _protect($l), '');
        },
        DISCARD_B => sub {
            my ($l, $r) = $get->(@_);
            ++$rl;
-           $out .= sprintf $line => '', $rl, '', _protect($r);
+           $out .= $line->('disc_b', '', $rl, '', _protect($r));
        },
        CHANGE    => sub {
            my ($l, $r) = $get->(@_);
@@ -181,8 +229,8 @@ sub _internal_diff {
                            append_close => '#/ins#',
                           );
            ++$ll; ++$rl;
-           $out .= sprintf $line => $ll, $rl,
-             _retag(_protect($diff->[0])), _retag(_protect($diff->[1]));
+           $out .= $line->('change', $ll, $rl,
+                           _retag(_protect($diff->[0])), _retag(_protect($diff->[1])));
        },
       };
     $out .= "</table>\n";
